/*
 * week_day.c
 *
 *  Created on: 16/mar/2014
 *      Author: fb
 *
 *  This program shows on the GREEN LEDS the day of the week of a given date
 *  which is set through the KEY switches and shown on the 7-seg displays
 *
 */

#include "../Inc/config.h"
#include <stdint.h>

/*
 * Function: HexToSevenSeg
 * -----------------------
 * Converts each of the 4 hex digit of a 16-bit number into the corresponding representation on the 7-seg display
 * shown below and assembles them into a 32-bit word that can be used to control a set of 4x 7-seg displays present
 * in DE boards
 *         0
 *       -----
 *     5 |___| 1
 *     4 | 6 | 2
 *       -----
 *         3
 */

uint32_t HexToSevenSeg(unsigned int hex) {
	const uint8_t LUT_HEX_TO_SEVEN_SEG[16] = {
	    0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 0x7F, 0x67, /* 0-9 */
	    0x77, 0x7C, 0x39, 0x5E, 0x79, 0x71 };                       /* A,b,C,d,E,F */

	uint32_t data = LUT_HEX_TO_SEVEN_SEG[hex            & 15]        |
			           (LUT_HEX_TO_SEVEN_SEG[(hex >> 4) & 15] <<  8) |
			           (LUT_HEX_TO_SEVEN_SEG[(hex >> 8) & 15] << 16) |
			           (LUT_HEX_TO_SEVEN_SEG[(hex >>12) & 15] << 24);

    return data;
}

/*
 * Function: binToDec
 *
 * Converts a number to its decimal representation. Each decimal digit is coded using 4 bits.
 * If the number cannot be represented using 8 decimal digits, the functions returns -1
 *
 *
*/

int32_t binToDec(uint32_t bin) {
	int32_t dec=0;
	uint8_t i;
	for(i=0; i<8; i++){
		dec = dec | ((bin % 10) << (4*i));
		bin = bin/10;
	}
	if (bin > 0) return -1;
	return dec;
}


int main() {

	uint32_t day=1, month=1, year=2014, week=4; /* 0..6 <=> SAT..FRI */
	uint8_t buttons, sel=0;

	/* Clear edge capture register of the PUSHBUTTONS Parallel Port
	 * Writing to this register clears all register bits */
	PUHBUTTONS_EDGE_CAPTURE_REG = 0;

	while(1) {

		buttons = PUHBUTTONS_EDGE_CAPTURE_REG;
		PUHBUTTONS_EDGE_CAPTURE_REG = 0;

		if(buttons & (1<<KEY_SHIFT)) {
			sel = (sel + 1) % 3;
		}

		switch(sel) {
			case 0: /* day */
				RED_LEDS_DATA_REG = 1 << RED_LED_DAY;
				if((buttons & (1<<KEY_UP) && (day < 31))) { day++; }
				if((buttons & (1<<KEY_DOWN) && (day > 1))) { day--; }
				break;
			case 1: /* month */
				RED_LEDS_DATA_REG = 1 << RED_LED_MONTH;
				if((buttons & (1<<KEY_UP) && (month < 12))) { month++; }
				if((buttons & (1<<KEY_DOWN) && (month > 1))) { month--; }
				break;
			case 2: /* year */
				RED_LEDS_DATA_REG = 1 << RED_LED_YEAR;
				if(buttons & (1<<KEY_UP)) { year++; }
				if(buttons & (1<<KEY_DOWN)) { year--; }
				break;
		}

		/* Show date of the 7-seg displays */
		HEX7_HEX4_DATA_REG = HexToSevenSeg((binToDec(day) << 8) | binToDec(month));
		HEX3_HEX0_DATA_REG = HexToSevenSeg(binToDec(year));
		/* Evaluate week day */
		if (month <= 2) {
			week = (year+31*(month-1)+day+(year-1)/4-3*((year+99)/100)/4)%7; }
		else {
			week = (year+31*(month-1)+day-(4*month+23)/10+year/4-(3*(year/100+1)/4))%7;  }
		/* Show week day MON...SUN on LEDG6...LEDG0 */
		GREEN_LEDS_DATA_REG = (1 << (6 - (week + 5)%7)) | (1 << 7);
	}
}
